@@ -113,7 +113,10 @@ def start(cif, scheduler):
         Operation(
             sampling.HostMonitor(cif=cif),
             config.getint('vars', 'host_sample_stats_interval'),
-            scheduler),
+            scheduler,
+            timeout=config.getint('vars', 'host_sample_stats_interval'),
+            exclusive=True,
+            discard=False),
 
         Operation(
             containersconnection.monitor,
@@ -145,7 +148,8 @@ class Operation(object):
 
     _log = logging.getLogger("virt.periodic.Operation")
 
-    def __init__(self, func, period, scheduler, timeout=0, executor=None):
+    def __init__(self, func, period, scheduler, timeout=0, executor=None,
+                 exclusive=False, discard=True):
         """
         parameters:
 
@@ -156,12 +160,22 @@ class Operation(object):
         timeout: same meaning of Executor.dispatch
         scheduler: Scheduler instance to use
         executor: Executor instance to use
+        exclusive: boolean flag to control the exclusiveness of the operation.
+                   Exclusive operations are scheduled again when and only when
+                   completed (conservative approach). Non-exclusive operations
+                   are scheduled again just after being dispatched to the
+                   executor (optimistic approach).
+                   The operations are non-exclusive by default.
+        discard: boolean flag to pass to the underlying executor.
+                 See the documentation of the 'Executor.dispatch' method.
         """
         self._func = func
         self._period = period
         self._timeout = _timeout_from(period) if timeout == 0 else timeout
         self._scheduler = scheduler
         self._executor = _executor if executor is None else executor
+        self._exclusive = exclusive
+        self._discard = discard
         self._lock = threading.Lock()
         self._running = False
         self._call = None
@@ -172,7 +186,7 @@ class Operation(object):
                 raise AssertionError("Operation already running")
             self._log.debug("starting operation %s", self._func)
             self._running = True
-            # we do _dispatch instead of _step here to have some
+            # we do _dispatch instead of _reschedule here to have some
             # data as soon as possibile
             self._dispatch()
 
@@ -190,8 +204,11 @@ class Operation(object):
             self._func()
         except Exception:
             self._log.exception("%s operation failed", self._func)
+        finally:
+            if self._exclusive:
+                self._reschedule()
 
-    def _step(self):
+    def _reschedule(self):
         """
         Schedule a next call of `func'.
         """
@@ -212,12 +229,13 @@ class Operation(object):
         """
         self._call = None
         try:
-            self._executor.dispatch(self, self._timeout)
+            self._executor.dispatch(self, self._timeout, discard=self._discard)
         except executor.TooManyTasks:
             self._log.warning('could not run %s, executor queue full',
                               self._func)
         finally:
-            self._step()
+            if not self._exclusive:
+                self._reschedule()
 
     def __repr__(self):
         return '<Operation action=%s at 0x%x>' % (
